{% extends 'admin/change_list.html' %}


{# --- 适配simpleui风格 --- #}


{% block object-tools %}
    {{ block.super }}

    <div>

        <div>
            <span
                style="
                    font-weight: 700;
                    margin:20px 20px 20px 20px;
                    font-size: 20px!important;
                    color: #666;
                "
            >
                {{ opts.verbose_name_plural }}
            </span>

            <a href="export_all_excel/" target="_blank">
               <button type="button" class="el-button el-button--default el-button--small"

                        style="
                        float: right;
                        margin-bottom: 10px;
                        margin-top: 10px;
                        margin-right: 12px;"

                       data-name="export_all_excel"><!----><!---->
                    <span class="fas fa-download"></span>
                    <span>
                        全部导出
                    </span>
                </button>
            </a>

            <a href="import-csv/">
                <button type="button" class="el-button el-button--default el-button--small"

                        style="
                        float: right;
                        margin-bottom: 10px;
                        margin-top: 10px;
                        margin-right: 12px;"

                        data-name="export_as_csv">
                    <i class="fas fa-upload"></i>
                    <span>
                        导入数据
                    </span>
                </button>
            </a>

        </div>
    </div>
{% endblock %}

