# {{ introduction_title }}



## 简要描述

{{ introduction_summary }}



## 请求URL

- `{{ request_url }}`



## 请求方式

- {{ context_request_type }}



## 请求参数

- 基本参数: 参考[API文档格式说明](https://gitee.com/LoongText/haiNanJieBangGuaShuaiPingTai_dj2138/wikis/%E5%89%8D%E5%8F%B0%E9%A1%B5%E9%9D%A2/%E4%B8%93%E5%AE%B6%E4%BA%BA%E6%89%8D/API%E6%96%87%E6%A1%A3%E6%A0%BC%E5%BC%8F%E8%AF%B4%E6%98%8E)



## 请求示例

```
{{ introduction_url }}
```



## 返回示例

```

{{ example_data }}

```



## 返回参数说明

{{ parameters_explain }}



## 备注

- 无






